//
//  myGraph.java
//  detectionContours
//
//  Created by Thomas Jannaud on 29/12/09.
//  Copyright 2009 Jannaud's Technical Company. All rights reserved.
//

import java.util.HashSet;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Collection;

public class myGraph {
	private int size = 0, index_source = -1, index_sink = -1;
	HashMap<Integer, HashMap<Integer, Integer>> edges = new HashMap<Integer, HashMap<Integer, Integer>>(); // destination edge / weight
	HashMap<myPoint, Integer> vertices = new HashMap<myPoint, Integer>();
	HashMap<Integer, myPoint> vertices_by_index = new HashMap<Integer, myPoint>();
	
	public myGraph() {}
	public int size() {return size;}
	public void setSource(int s) {index_source = s;}
	public void setSink(int s) {index_sink = s;}
	
	public void addEdge(int start, int end, int weight) {
		(edges.get(start)).put(end, weight);
	}
	
	public void addVertices(int n) {
		for(int i = 0; i < n; i++)
			addVertex(null);
	}
	
	public int addVertex(myPoint p) {
		if (p != null) {
			vertices.put(p, size);
			vertices_by_index.put(size, p);
		}
		edges.put(size, new HashMap<Integer, Integer>());
		size++;
		return (size - 1);
	}
	
	public int addOrRecognize(myPoint p) {
		if (vertices.containsKey(p))
			return vertices.get(p);
		else
			return addVertex(p);
	}
	
	public void print() {
		System.out.println("graphe of size " + size);
		for(int i = 0; i < size; i++) {
			System.out.println("\nneighbours of " + i);
			HashMap<Integer, Integer> h = edges.get(i);
			Collection<Integer> c = h.keySet();
			for(Integer j : c)
				System.out.println(i + " -> " + j + " (" + h.get(j) + ")");
		}
		System.out.println("\n\nList of vertices");
		Collection<myPoint> c = vertices.keySet();
		for(myPoint p : c) 
			System.out.println(vertices.get(p) + " : (" + p.x + ", " + p.y + ")");
	}
	
	// ------------------
	// MAX FLOW / MIN CUT
	// ------------------
	
	public pair<Collection<myPoint>, Integer> minCut() {
		HashSet<Integer> marked = new HashSet<Integer>();
		HashMap<myPoint, Integer> flow = computeMaxFlow();
		LinkedList<Integer> queue = new LinkedList<Integer>();
		queue.addFirst(index_source);
		marked.add(index_source);
		while(!queue.isEmpty()) {
			int start = queue.removeFirst();
			HashMap<Integer, Integer> h = edges.get(start);
			Collection<Integer> c = h.keySet();
			for(Integer destination : c) {
				int capa = h.get(destination);
				int flot = flow.get(new myPoint(start, destination));
				if (flot < capa && !marked.contains(destination)) {
					marked.add(destination);
					queue.addLast(destination);
				}
			}
		}
		
		int cutVal = 0;
		HashSet<myPoint> cut = new HashSet<myPoint>();
		for(Integer i : marked) {
			HashMap<Integer, Integer> h = edges.get(i);
			Collection<Integer> c = h.keySet();
			myPoint p_i = vertices_by_index.get(i);
			for(Integer j : c) {
				if (!marked.contains(j)) {
					myPoint p_j = vertices_by_index.get(j);
					if (i == index_source) {
						if (p_j != null)
							cut.add(new myPoint(p_j.x, p_j.y, myPoint.CONTOURS));
						else
							System.out.println("ERROR in minCut, p_j should not be null when i is the source");
					}
					else if (i == index_sink) {
						System.out.println("ERROR in minCut, the sink should not be marked");
					}
					else if (p_i != null) {
						cut.add(new myPoint(p_i.x, p_i.y, myPoint.CONTOURS));
					}
					else if (p_j != null) {
						cut.add(new myPoint(p_j.x, p_j.y, myPoint.CONTOURS));
					}
					else
						System.out.println("ERROR in minCut, should not be there");
					cutVal += h.get(j);
				}
			}
		}
		return new pair<Collection<myPoint>, Integer>(cut, cutVal);
	}
	
	public HashMap<myPoint, Integer> computeMaxFlow() {
		HashMap<myPoint, Integer> flow = new HashMap<myPoint, Integer>(); // myPoint x,y stands for edge x->y
		HashMap<Integer, HashMap<Integer, Integer>> residual_capacity = new HashMap<Integer, HashMap<Integer, Integer>>(); // x->y->capacity
		
		// initialization
		for(int i = 0; i < size; i++)
			residual_capacity.put(i, new HashMap<Integer, Integer>());
		for(int i = 0; i < size; i++) {
			HashMap<Integer, Integer> h = edges.get(i);
			Collection<Integer> c = h.keySet();
			HashMap<Integer, Integer> capa_i = residual_capacity.get(i);
			for(Integer j : c) {
				flow.put(new myPoint(i, j), 0);
				flow.put(new myPoint(j, i), 0);
				capa_i.put(j, h.get(j));
				if (!residual_capacity.get(j).containsKey(i))
					residual_capacity.get(j).put(i, 0);
			}
		}
		
		while(true) {
			LinkedList<Integer> l = find_augmenting_path(residual_capacity);
			if (l == null)
				break;
			reset(l, flow, residual_capacity);
		}
		return flow;
	}
	
	private LinkedList<Integer> find_augmenting_path(HashMap<Integer, HashMap<Integer, Integer>> residual_capacity) {
		HashMap<Integer, Integer> ancestor = new HashMap<Integer, Integer>();
		
		// bfs
		LinkedList<Integer> queue = new LinkedList<Integer>();
		queue.addFirst(index_source);
		ancestor.put(index_source, -1);
		while(!queue.isEmpty()) {
			int start = queue.removeFirst();
			HashMap<Integer, Integer> h = residual_capacity.get(start);
			Collection<Integer> c = h.keySet();
			for(Integer destination : c) {
				int capa = h.get(destination);
				if (capa > 0 && !ancestor.containsKey(destination)) {
					ancestor.put(destination, start);
					queue.addLast(destination);
				}
			}
			if (start == index_sink)
				break;
		}
		
		// returned path : <s1 = source, s2, ... sn = sink>
		LinkedList<Integer> l = new LinkedList<Integer>();
		int v = index_sink;
		while(v != index_source && ancestor.containsKey(v)) {
			l.addFirst(v);
			v = ancestor.get(v);
		}
		l.addFirst(index_source);
		if (v == index_source)
			return l;
		else
			return null;
	}
	
	
	private void reset(LinkedList<Integer> augmenting_path, HashMap<myPoint, Integer> flow, HashMap<Integer, HashMap<Integer, Integer>> residual_capacity) {
		// determine minimum capacity along the path
		LinkedList<Integer> l = new LinkedList<Integer>();
		for(Integer i : augmenting_path)
			l.addLast(i);
		final int n = augmenting_path.size();
		int capa_min = 1000000;
		for(int i = 0; i < (n - 1); i++) {
			int a = augmenting_path.removeFirst();
			int b = augmenting_path.getFirst();
			int c = residual_capacity.get(a).get(b);
			capa_min = Math.min(c, capa_min);
		}
		
		augmenting_path = l;
		
		for(int i = 0; i < (n - 1); i++) {
			int a = augmenting_path.removeFirst();
			int b = augmenting_path.getFirst();
			int c1 = residual_capacity.get(a).get(b);
			int c2 = residual_capacity.get(b).get(a);
			residual_capacity.get(a).put(b, c1 - capa_min);
			residual_capacity.get(b).put(a, c2 + capa_min);
			int f1 = flow.get(new myPoint(a, b));
			int f2 = flow.get(new myPoint(b, a));
			flow.put(new myPoint(a, b), f1 + capa_min);
			flow.put(new myPoint(b, a), f2 - capa_min);
		}
	}
	
	private void affiche(Collection<Integer> c) {
		for(Integer i : c)
			System.out.print(i + " - ");
	}
	
	private void affiche(HashMap<myPoint, Integer> aMap) {
		Collection<myPoint> c = aMap.keySet();
		for(myPoint p : c)
			System.out.println(p.x + " -> " + p.y + " (" + aMap.get(p) + ")");
	}
	
	private void affiche2(HashMap<Integer, HashMap<Integer, Integer>> aMap) {
		Collection<Integer> c = aMap.keySet();
		for(Integer i : c) {
			HashMap<Integer, Integer> h = aMap.get(i);
			Collection<Integer> cc = h.keySet();
			for(Integer j : cc)
				System.out.println(i + "->" + j + " (" + h.get(j) + ")");
		}
	}	
	
	public static myGraph create_graphe(final Collection<myPoint> belt, myGreyPicture image, int W, int H) {
		// add a source and a sink, computes edges
		myGraph g = new myGraph();
		final int index_source = 0, index_sink = 1;
		g.addVertices(2);
		g.setSource(index_source);
		g.setSink(index_sink);
		myPoint.setComparator(myPoint.COMPARE_POINT);
		for(myPoint p : belt) {
			if (!p.goodPoint(W, H))
				continue;
			int index_p = g.addOrRecognize(p);
			for(int i = -1; i <= 1; i++)
				for(int j = -1; j <= 1; j++) {
					if (i == 0 && j == 0)
						continue;
					myPoint q = new myPoint(p.x + i, p.y + j);
					if (!q.goodPoint(W, H))
						g.addEdge(index_p, index_sink, 100);
					else {
						double grey1 = image.greyComponent(p.x, p.y);
						double grey2 = image.greyComponent(q.x, q.y);
						int val = (int) Math.round(100. * Math.exp(-8. * Math.abs(grey2 - grey1)));
						if (!belt.contains(q)) {
							// if inside/outside point of the belt touchs a point not in the belt, then it is the source/sink
							if (p.type == myPoint.INSIDE_BELT)
								g.addEdge(index_source, index_p, val);
							else
								g.addEdge(index_p, index_sink, val);
						}
						else {
							// find the index of the other point in the belt. If index doesn't exist, create it.
							int index_q = g.addOrRecognize(q);
							g.addEdge(index_p, index_q, val);
							g.addEdge(index_q, index_p, val);
						}
					}
				}
		}
		return g;
	}
}